@@ -132,10 +132,12 @@ GSList *at_util_parse_clcc(GAtResult *result)
 		if (g_at_result_iter_next_string(&iter, &str))
 			g_at_result_iter_next_number(&iter, &number_type);
 
-		call = g_try_new0(struct ofono_call, 1);
+		call = g_try_new(struct ofono_call, 1);
 		if (call == NULL)
 			break;
 
+		ofono_call_init(call);
+
 		call->id = id;
 		call->direction = dir;
 		call->status = status;
@@ -178,6 +180,7 @@ gboolean at_util_parse_reg_unsolicited(GAtResult *result, const char *prefix,
 		goto out;
 
 	switch (vendor) {
+	case OFONO_VENDOR_GOBI:
 	case OFONO_VENDOR_HUAWEI:
 	case OFONO_VENDOR_NOVATEL:
 		if (g_at_result_iter_next_unquoted_string(&iter, &str) == TRUE)
@@ -262,6 +265,7 @@ gboolean at_util_parse_reg(GAtResult *result, const char *prefix,
 			goto out;
 
 		switch (vendor) {
+		case OFONO_VENDOR_GOBI:
 		case OFONO_VENDOR_HUAWEI:
 		case OFONO_VENDOR_NOVATEL:
 			r = g_at_result_iter_next_unquoted_string(&iter, &str);
