@@ -54,11 +54,11 @@
 #include <ofono/sim.h>
 #include <ofono/stk.h>
 #include <ofono/sms.h>
-#include <ofono/ssn.h>
 #include <ofono/ussd.h>
 #include <ofono/voicecall.h>
 #include <ofono/gprs.h>
 #include <ofono/gprs-context.h>
+#include <ofono/gnss.h>
 
 #include <drivers/atmodem/vendor.h>
 #include <drivers/atmodem/sim-poll.h>
@@ -78,19 +78,36 @@ struct phonesim_data {
 struct gprs_context_data {
 	GAtChat *chat;
 	char *interface;
+	enum ofono_gprs_proto proto;
 };
 
 static void at_cgact_up_cb(gboolean ok, GAtResult *result, gpointer user_data)
 {
 	struct cb_data *cbd = user_data;
-	ofono_gprs_context_up_cb_t cb = cbd->cb;
+	ofono_gprs_context_cb_t cb = cbd->cb;
 	struct ofono_gprs_context *gc = cbd->user;
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 	struct ofono_error error;
 
 	decode_at_error(&error, g_at_result_final_response(result));
-	cb(&error, ok ? gcd->interface : NULL, FALSE,
-			NULL, NULL, NULL, NULL, cbd->data);
+
+	if (ok == FALSE)
+		goto done;
+
+	ofono_gprs_context_set_interface(gc, gcd->interface);
+
+	if (gcd->proto == OFONO_GPRS_PROTO_IP ||
+			gcd->proto == OFONO_GPRS_PROTO_IPV4V6)
+		ofono_gprs_context_set_ipv4_address(gc, NULL, FALSE);
+
+	if (gcd->proto == OFONO_GPRS_PROTO_IPV6 ||
+			gcd->proto == OFONO_GPRS_PROTO_IPV4V6) {
+		ofono_gprs_context_set_ipv6_address(gc, "fe80::1");
+		ofono_gprs_context_set_ipv6_prefix_length(gc, 10);
+	}
+
+done:
+	cb(&error, cbd->data);
 }
 
 static void at_cgact_down_cb(gboolean ok, GAtResult *result, gpointer user_data)
@@ -105,16 +122,32 @@ static void at_cgact_down_cb(gboolean ok, GAtResult *result, gpointer user_data)
 
 static void phonesim_activate_primary(struct ofono_gprs_context *gc,
 				const struct ofono_gprs_primary_context *ctx,
-				ofono_gprs_context_up_cb_t cb, void *data)
+				ofono_gprs_context_cb_t cb, void *data)
 {
 	struct gprs_context_data *gcd = ofono_gprs_context_get_data(gc);
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[OFONO_GPRS_MAX_APN_LENGTH + 128];
-	int len;
+	int len = 0;
 
 	cbd->user = gc;
+	gcd->proto = ctx->proto;
 
-	len = snprintf(buf, sizeof(buf), "AT+CGDCONT=%u,\"IP\"", ctx->cid);
+	switch (ctx->proto) {
+	case OFONO_GPRS_PROTO_IP:
+		len = snprintf(buf, sizeof(buf), "AT+CGDCONT=%u,\"IP\"",
+				ctx->cid);
+		break;
+
+	case OFONO_GPRS_PROTO_IPV6:
+		len = snprintf(buf, sizeof(buf), "AT+CGDCONT=%u,\"IPV6\"",
+				ctx->cid);
+		break;
+
+	case OFONO_GPRS_PROTO_IPV4V6:
+		len = snprintf(buf, sizeof(buf), "AT+CGDCONT=%u,\"IPV4V6\"",
+				ctx->cid);
+		break;
+	}
 
 	if (ctx->apn)
 		snprintf(buf + len, sizeof(buf) - len - 3, ",\"%s\"",
@@ -132,7 +165,7 @@ static void phonesim_activate_primary(struct ofono_gprs_context *gc,
 error:
 	g_free(cbd);
 
-	CALLBACK_WITH_FAILURE(cb, NULL, 0, NULL, NULL, NULL, NULL, data);
+	CALLBACK_WITH_FAILURE(cb, data);
 }
 
 static void phonesim_deactivate_primary(struct ofono_gprs_context *gc,
@@ -143,9 +176,6 @@ static void phonesim_deactivate_primary(struct ofono_gprs_context *gc,
 	struct cb_data *cbd = cb_data_new(cb, data);
 	char buf[128];
 
-	if (cbd == NULL)
-		goto error;
-
 	cbd->user = gc;
 
 	snprintf(buf, sizeof(buf), "AT+CGACT=0,%u", id);
@@ -154,7 +184,6 @@ static void phonesim_deactivate_primary(struct ofono_gprs_context *gc,
 				at_cgact_down_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, data);
@@ -273,14 +302,10 @@ static void phonesim_ctm_query(struct ofono_ctm *ctm,
 
 	DBG("");
 
-	if (!cbd)
-		goto error;
-
 	if (g_at_chat_send(chat, "AT+PTTY?", ptty_prefix,
 				ctm_query_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	g_free(cbd);
 
 	CALLBACK_WITH_FAILURE(cb, 0, data);
@@ -305,16 +330,12 @@ static void phonesim_ctm_set(struct ofono_ctm *ctm, ofono_bool_t enable,
 
 	DBG("");
 
-	if (!cbd)
-		goto error;
-
 	snprintf(buf, sizeof(buf), "AT+PTTY=%d", enable);
 
 	if (g_at_chat_send(chat, buf, none_prefix,
 				ctm_set_cb, cbd, g_free) > 0)
 		return;
 
-error:
 	CALLBACK_WITH_FAILURE(cb, data);
 	g_free(cbd);
 }
@@ -637,6 +658,9 @@ static void phonesim_post_sim(struct ofono_modem *modem)
 					"atmodem", data->chat);
 
 	ofono_call_forwarding_create(modem, 0, "atmodem", data->chat);
+
+	if (!data->calypso)
+		ofono_sms_create(modem, 0, "atmodem", data->chat);
 }
 
 static void phonesim_post_online(struct ofono_modem *modem)
@@ -660,21 +684,18 @@ static void phonesim_post_online(struct ofono_modem *modem)
 
 	ofono_call_meter_create(modem, 0, "atmodem", data->chat);
 	ofono_call_barring_create(modem, 0, "atmodem", data->chat);
-	ofono_ssn_create(modem, 0, "atmodem", data->chat);
 	ofono_call_volume_create(modem, 0, "atmodem", data->chat);
 
-	if (!data->calypso) {
-		ofono_sms_create(modem, 0, "atmodem", data->chat);
+	if (!data->calypso)
 		ofono_cbs_create(modem, 0, "atmodem", data->chat);
-	}
-
-	gprs = ofono_gprs_create(modem, 0, "atmodem", data->chat);
 
 	gc1 = ofono_gprs_context_create(modem, 0, "phonesim", data->chat);
+	gprs = ofono_gprs_create(modem, 0, "atmodem", data->chat);
+	gc2 = ofono_gprs_context_create(modem, 0, "phonesim", data->chat);
+
 	if (gprs && gc1)
 		ofono_gprs_add_context(gprs, gc1);
 
-	gc2 = ofono_gprs_context_create(modem, 0, "phonesim", data->chat);
 	if (gprs && gc2)
 		ofono_gprs_add_context(gprs, gc2);
 
@@ -682,6 +703,7 @@ static void phonesim_post_online(struct ofono_modem *modem)
 	if (mw)
 		ofono_message_waiting_register(mw);
 
+	ofono_gnss_create(modem, 0, "atmodem", data->chat);
 }
 
 static struct ofono_modem_driver phonesim_driver = {
