@@ -134,6 +134,7 @@ enum ofono_atom_type {
 	OFONO_ATOM_TYPE_EMULATOR_HFP,
 	OFONO_ATOM_TYPE_LOCATION_REPORTING,
 	OFONO_ATOM_TYPE_GNSS,
+	OFONO_ATOM_TYPE_CDMA_SMS,
 };
 
 enum ofono_atom_watch_condition {
@@ -270,6 +271,9 @@ int __ofono_voicecall_tone_send(struct ofono_voicecall *vc,
 				ofono_voicecall_tone_cb_t cb, void *user_data);
 void __ofono_voicecall_tone_cancel(struct ofono_voicecall *vc, int id);
 
+struct ofono_call *__ofono_voicecall_find_call_with_status(
+				struct ofono_voicecall *vc, int status);
+
 #include <ofono/sms.h>
 
 struct sms;
@@ -465,3 +469,9 @@ void __ofono_gprs_provision_free_settings(
 
 #include <ofono/emulator.h>
 #include <ofono/gnss.h>
+#include <ofono/cdma-sms.h>
+#include <ofono/private-network.h>
+
+void __ofono_private_network_release(int id);
+ofono_bool_t __ofono_private_network_request(ofono_private_network_cb_t cb,
+						int *id, void *data);
